import unittest

from CBDMultipleOutput.Source.CBD import *
from CBDMultipleOutput.Source.CBD2LatexCompiler import *
from CBDMultipleOutput.Source.CBD2DevsCompiler import *

class CompilerTestCase(unittest.TestCase):
	def setUp(self):
		self.CBD = CBD("CBD_for_block_under_test")

	def testNoLoop(self):               
		self.CBD.addBlock(ConstantBlock(block_name="c1", value=1.0))
		self.CBD.addBlock(ConstantBlock(block_name="c2", value=2.0))
		self.CBD.addBlock(ConstantBlock(block_name="c3", value=3.0))
		self.CBD.addBlock(ConstantBlock(block_name="c4", value=4.0))
		self.CBD.addBlock(NegatorBlock(block_name="n1"))
		self.CBD.addBlock(AdderBlock(block_name="a1"))
		self.CBD.addBlock(AdderBlock(block_name="a2"))
		self.CBD.addBlock(ProductBlock(block_name="p1"))

		self.CBD.addConnection("c1", "a1")
		self.CBD.addConnection("c2", "a1")
		self.CBD.addConnection("c3", "a2")
		self.CBD.addConnection("c4", "n1")
		self.CBD.addConnection("n1", "a2")
		self.CBD.addConnection("a1", "p1")
		self.CBD.addConnection("a2", "p1")

		devsCompiler = CBD2DevsCompiler(self.CBD)
		devsCompiler.compileToDevs(filename="cbd.py")
		compiler = CBD2LatexCompiler(self.CBD)
		compiler.compileToLatex(filename="cbd.tex")

		self.assertEquals(True, True)

	def testLinearLoop(self):
		self.CBD.addBlock(ConstantBlock(block_name="c1", value=5.0))
		self.CBD.addBlock(ConstantBlock(block_name="c2", value=2.0))
		self.CBD.addBlock(ConstantBlock(block_name="c3", value=2.0))
		self.CBD.addBlock(ProductBlock(block_name="p1"))
		self.CBD.addBlock(ProductBlock(block_name="p2"))
		self.CBD.addBlock(NegatorBlock(block_name="n1"))
		self.CBD.addBlock(AdderBlock(block_name="a1"))

		self.CBD.addConnection("c1","p1")
		self.CBD.addConnection("c2","p1")
		self.CBD.addConnection("c3","p2")
		self.CBD.addConnection("a1","p2")
		self.CBD.addConnection("p1","n1")
		self.CBD.addConnection("n1","a1")
		self.CBD.addConnection("p2","a1")

		compiler2 = CBD2DevsCompiler(self.CBD)
		compiler2.compileToDevs(filename="cbd_2.py")
		compiler = CBD2LatexCompiler(self.CBD)
		compiler.compileToLatex(filename="cbd_2.tex")

	def testMultipleLinearLoops(self):
		self.CBD.addBlock(ConstantBlock(block_name="c1", value=1.0))
		self.CBD.addBlock(ConstantBlock(block_name="c2", value=2.0))
		self.CBD.addBlock(ConstantBlock(block_name="c3", value=3.0))
		self.CBD.addBlock(AdderBlock(block_name="a1"))
		self.CBD.addBlock(AdderBlock(block_name="a2"))
		self.CBD.addBlock(AdderBlock(block_name="a3"))
		self.CBD.addBlock(AdderBlock(block_name="a4"))
		self.CBD.addBlock(AdderBlock(block_name="a5"))
		self.CBD.addBlock(AdderBlock(block_name="a6"))
		self.CBD.addBlock(ProductBlock(block_name="p"))
		self.CBD.addBlock(NegatorBlock(block_name="n"))

		self.CBD.addConnection("c1", "a1")
		self.CBD.addConnection("c2", "a2")
		self.CBD.addConnection("c3","p")
		self.CBD.addConnection("a1", "a3")
		self.CBD.addConnection("a2", "a3")
		self.CBD.addConnection("a2", "p")
		self.CBD.addConnection("a3", "a1")
		self.CBD.addConnection("a3", "a2")
		self.CBD.addConnection("a4", "a6")
		self.CBD.addConnection("a5", "a6")
		self.CBD.addConnection("a6", "a4")
		self.CBD.addConnection("a6", "n")
		self.CBD.addConnection("p", "a4")
		self.CBD.addConnection("p", "a5")
		self.CBD.addConnection("n", "a5")

		devsCompiler = CBD2DevsCompiler(self.CBD)
		devsCompiler.compileToDevs(filename="cbd_3.py")
		compiler = CBD2LatexCompiler(self.CBD)
		compiler.compileToLatex(filename="cbd_3.tex")


		self.assertEquals(True, True)

	def testBasicDerivator(self):
		inputSignal = CBD("inputSignal", output_ports=["OUT1"])
		inputSignal.addBlock(ConstantBlock(block_name="c1", value=2.0))
		inputSignal.addBlock(ConstantBlock(block_name="c2", value=5.0))
		inputSignal.addBlock(TimeBlock(block_name="t"))
		inputSignal.addBlock(ProductBlock(block_name="p"))
		inputSignal.addBlock(AdderBlock(block_name="a"))
		inputSignal.addConnection("c1", "p")
		inputSignal.addConnection("t", "p")
		inputSignal.addConnection("p", "a")
		inputSignal.addConnection("c2", "a")
		inputSignal.addConnection("a", "OUT1")

		self.CBD.addBlock(ConstantBlock(block_name="c3", value=1.0))
		self.CBD.addBlock(inputSignal)
		self.CBD.addBlock(DerivatorBlock(block_name="deriv"))

		self.CBD.addConnection("c3", "deriv", input_port_name="delta_t")
		self.CBD.addConnection("inputSignal", "deriv")

		devsCompiler = CBD2DevsCompiler(self.CBD)
		devsCompiler.compileToDevs(filename="cbd_4.py")
		compiler = CBD2LatexCompiler(self.CBD)
		compiler.compileToLatex(filename="cbd_4.tex")


		self.assertEquals(True, True)

	def testBasicIntegrator(self):
		inputSignal = CBD("inputSignal", output_ports=["OUT1"])
		inputSignal.addBlock(ConstantBlock(block_name="c1", value=10.0))
		inputSignal.addBlock(ConstantBlock(block_name="c2", value=-3.0))
		inputSignal.addBlock(TimeBlock(block_name="t"))
		inputSignal.addBlock(ProductBlock(block_name="p"))
		inputSignal.addBlock(AdderBlock(block_name="a"))
		inputSignal.addConnection("c1", "p")
		inputSignal.addConnection("t", "p")
		inputSignal.addConnection("p", "a")
		inputSignal.addConnection("c2", "a")
		inputSignal.addConnection("a", "OUT1")

		self.CBD.addBlock(ConstantBlock(block_name="delta", value=0.001))
		self.CBD.addBlock(inputSignal)
		self.CBD.addBlock(IntegratorBlock(block_name="integr"))

		self.CBD.addConnection("delta", "integr", input_port_name="delta_t")
		self.CBD.addConnection("inputSignal", "integr")

		devsCompiler = CBD2DevsCompiler(self.CBD)
		devsCompiler.compileToDevs(filename="cbd_5.py")
		compiler = CBD2LatexCompiler(self.CBD)
		compiler.compileToLatex(filename="cbd_5.tex")
		self.assertEquals(True,True)

	def testLinkedIntegrator(self):
		inputSignal = CBD("inputSignal", output_ports=["OUT1"])
		inputSignal.addBlock(ConstantBlock(block_name="c1", value=10.0))
		inputSignal.addBlock(ConstantBlock(block_name="c2", value=-3.0))
		inputSignal.addBlock(TimeBlock(block_name="t"))
		inputSignal.addBlock(ProductBlock(block_name="p"))
		inputSignal.addBlock(AdderBlock(block_name="a"))
		inputSignal.addConnection("c1", "p")
		inputSignal.addConnection("t", "p")
		inputSignal.addConnection("p", "a")
		inputSignal.addConnection("c2", "a")
		inputSignal.addConnection("a", "OUT1")

		self.CBD.addBlock(ConstantBlock(block_name="delta", value=0.001))
		self.CBD.addBlock(inputSignal)
		self.CBD.addBlock(IntegratorBlock(block_name="integr"))

		self.CBD.addConnection("delta", "integr", input_port_name="delta_t")
		self.CBD.addConnection("inputSignal", "integr")

		self.CBD.addBlock(ConstantBlock(block_name="c3", value=-1.0))
		self.CBD.addBlock(IntegratorBlock(block_name="integr2"))
		self.CBD.addConnection("delta", "integr2", input_port_name="delta_t")
		self.CBD.addBlock(AdderBlock(block_name="a2"))
		self.CBD.addConnection("c3", "a2")
		self.CBD.addConnection("integr", "a2")
		self.CBD.addConnection("a2", "integr2")

		devsCompiler = CBD2DevsCompiler(self.CBD)
		devsCompiler.compileToDevs(filename="cbd_6.py")
		compiler = CBD2LatexCompiler(self.CBD)
		compiler.compileToLatex(filename="cbd_6.tex")


		self.assertEquals(True, True)